package com.martico.portlet.util;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Locale;

import javax.portlet.PortletContext;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;

import com.itextpdf.text.BadElementException;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Image;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.ColumnText;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfPageEventHelper;
import com.itextpdf.text.pdf.PdfWriter;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.ServiceContextFactory;
import com.liferay.portal.theme.ThemeDisplay;

public abstract class AbstractPdfWriter<T> {

	protected static final String TREBUCHET_MS = "Trebuchet MS";
	protected ThemeDisplay themeDisplay;
	protected ServiceContext serviceContext;
	protected PortletContext portletContext;
	protected PdfWriter writer;
	protected Document document;
	protected ArrayList<Long> idList;
	Locale locale;

	protected AbstractPdfWriter(ResourceRequest resourceRequest, ResourceResponse resourceResponse,String fileName) throws PortalException, SystemException, DocumentException,
			IOException {
		this.serviceContext = ServiceContextFactory.getInstance(resourceRequest);
		this.themeDisplay = (ThemeDisplay) resourceRequest.getAttribute(WebKeys.THEME_DISPLAY);
		this.portletContext = resourceRequest.getPortletSession().getPortletContext();
		writer = PdfWriter.getInstance(createDocument(), resourceResponse.getPortletOutputStream());
		HeaderFooter event = new HeaderFooter(portletContext);
		writer.setPageEvent(event);
		resourceResponse.setContentType("application/pdf");
		resourceResponse.setProperty("Content-Disposition", "filename=\"" + fileName + "\"");
		idList = new ArrayList<Long>();
		locale = themeDisplay.getLocale();
	}

	public abstract void write(T e) throws Exception;
	
	private Document createDocument() {
		document = new Document();
		document.setPageSize(PageSize.A4);
		document.setMargins(86, 56, 60, 60);
		document.setMarginMirroring(true);
		return document;
	}

	protected static Paragraph createParagraph(String label, String value) {
		Paragraph paragraph = new Paragraph();
		paragraph.setAlignment(Chunk.ALIGN_JUSTIFIED);
		paragraph.setKeepTogether(true);
		Chunk chunk = new Chunk(label, FontFactory.getFont(TREBUCHET_MS, 8.5F, Font.BOLD, new BaseColor(0)));
		paragraph.add(chunk);
		paragraph.add(Chunk.NEWLINE);
		chunk = new Chunk(value, FontFactory.getFont(TREBUCHET_MS, 8.5F));
		paragraph.add(chunk);
		paragraph.add(Chunk.NEWLINE);
		paragraph.add(Chunk.NEWLINE);
		return paragraph;
	}

	protected static Paragraph createTitle(String title, int align) {
		Paragraph p = new Paragraph();
		Chunk chunk = new Chunk(title, FontFactory.getFont(TREBUCHET_MS, 15.5F, Font.BOLD, new BaseColor(0)));
		p.add(chunk);
		p.setAlignment(align);
		p.setSpacingBefore(10);
		p.setSpacingAfter(15);
		return p;
	}
	
	protected static Phrase createTitle(String title) {
		Phrase p = new Phrase();
		Chunk chunk = new Chunk(title, FontFactory.getFont(TREBUCHET_MS, 15.5F, Font.BOLD, new BaseColor(39, 172, 229)));
		p.add(chunk);
		return p;
	}
	
	protected static Paragraph createSubtitle(String title) {
		Paragraph p = new Paragraph();
		Chunk chunk = new Chunk(title, FontFactory.getFont(TREBUCHET_MS, 10F, Font.BOLD, new BaseColor(39, 172, 229)));
		p.add(chunk);

		p.setSpacingBefore(15);
		return p;
	}
	
	protected static Paragraph createFormField(String label, String content) {
		Paragraph p;
		Chunk chunk;
		p = new Paragraph();
		chunk = new Chunk(label + ": ", FontFactory.getFont(TREBUCHET_MS, 8.5F, Font.BOLD, new BaseColor(0)));
		p.add(chunk);
		if (content == null) content = "";
		chunk = new Chunk(content, FontFactory.getFont(TREBUCHET_MS, 8.5F));
		p.add(chunk);
		return p;
	}
	
	public void close() {
		document.close();
	}

	public void open() {
		document.open();
	}

	class HeaderFooter extends PdfPageEventHelper {

		PortletContext portletContext;

		public HeaderFooter(PortletContext portletContext) {
			super();
			this.portletContext = portletContext;
		}

		public void onEndPage(PdfWriter writer, Document document) {
			// Cabecera
			Image logo;
			try {
				logo = Image.getInstance(portletContext.getResource("/img/logo-martico.jpg"));
				ColumnText.showTextAligned(writer.getDirectContent(), Element.ALIGN_RIGHT, new Phrase(new Chunk(logo, 50, 50)), 250, 830, 180);
				ColumnText.showTextAligned(writer.getDirectContent(), Element.ALIGN_LEFT, createTitle(themeDisplay.getScopeGroupName()), 380, 790, 0);

			}
			catch (BadElementException e) {
				e.printStackTrace();
			}
			catch (MalformedURLException e) {
				e.printStackTrace();
			}
			catch (IOException e) {
				e.printStackTrace();
			} catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	protected PdfPCell addNormalCell(String content, int align, boolean single) {
		PdfPCell table_cell;
		table_cell= new PdfPCell(new Phrase(content,FontFactory.getFont(TREBUCHET_MS, 8.5F)));
		table_cell.setHorizontalAlignment(align);
		if (content.isEmpty()) {
			table_cell.setBorder(Rectangle.TOP);
		}
		if (single) {
			table_cell.setBorder(Rectangle.BOTTOM | Rectangle.RIGHT | Rectangle.LEFT);
		}
		return table_cell;
	}

	protected PdfPCell addTitleCell(String content, int align, boolean single) {
		PdfPCell table_cell;
		table_cell= new PdfPCell(new Phrase(content,FontFactory.getFont(TREBUCHET_MS, 8.5F,
				Font.BOLD)));
		table_cell.setHorizontalAlignment(align); 
		if (single) {
			table_cell.setBorder(Rectangle.TOP | Rectangle.RIGHT | Rectangle.LEFT);
		}
		return table_cell;
	}
	
	protected PdfPTable createPDFTable(int tablePos) {
		PdfPTable table = new PdfPTable(tablePos);
		table.setWidthPercentage(100);
		table.setSpacingBefore(10);
		return table;
	}
}
